{% comment %}
Copyright (c) Siemens AG, 2013

This file is part of MANTIS.  MANTIS is free software: you can
redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation; either version 2
of the License, or(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51
Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
{% endcomment %}


{% with marking_count=object.marking_thru.count %}

<div class="grp-module {% if marking_count > 0 %}grp-collapse{% endif %}">
<h2 class="grp-collapse-handler">
    {{ marking_count }} marking{% if marking_count != 1 %}s{% endif %}
</h2>
                {% for k in object.marking_thru.all %}

                    <div class='grp-row'>

                        <a href={% url 'url.dingos.view.infoobject' k.marking.pk %}>{{ k.marking.identifier.namespace.uri }}:{{k.marking.name}}</a>

                    </div>

                {% endfor %}
            </table>

    </div>

{%  endwith %}



